# -*- coding: utf-8 -*-
# moebel.py

## Ueberarbeitung zur Korrektur der Darstellung:
## Veraendernde Methoden greifen nur auf die
## Update-Methoden Zeichne() und Loesche() zu.

from grafikfenster import *
from math import radians

from abc import ABC,abstractmethod # abstrakte Klasse realisieren

### ---------------------------------------------------
class Moebel(ABC):
    """Klasse Moebel
    Oberklasse für das Zeichnen und Bearbeiten von
    Möbel-Symbolen fuer den Raumplaner"""
    
    def __init__(self,
                 xPos,
                 yPos,
                 breite,
                 tiefe,
                 winkel,
                 farbe,
                 sichtbar):
        """Konstruktor mit vordefinierten Parametern
        ist bei Moebel nicht sinnvoll"""
        self.__x=xPos
        self.__y=yPos
        self.__b=breite
        self.__t=tiefe
        self.__w=winkel
        self.__f=farbe
        self.__s=sichtbar
        if sichtbar: self.Zeichne()

    ### -----  properties -----
    ### x
    def GibXPosition(self):
        return self.__x
    def BewegeHorizontal(self, weite):
        """Veraendernde Methode fuer die x-Position"""
        self.Loesche()
        self.__x += weite
        self.Zeichne()
    x = property(GibXPosition,BewegeHorizontal)

    ### y
    def GibYPosition(self):
        return self.__y
    def BewegeVertikal(self, weite):
        """Veraendernde Methode fuer die y-Position"""
        self.Loesche()
        self.__y += weite
        self.Zeichne()
    y = property(GibYPosition,BewegeVertikal)

    ### b
    def GibBreite(self):
        return self.__b
    def AendereBreite(self, neueBreite):
        """Veraendernde Methode fuer die Breite"""
        self.Loesche()
        self.__b = neueBreite
        self.Zeichne()
    b = property(GibBreite,AendereBreite)

    ### t
    def GibTiefe(self):
        return self.__t
    def AendereTiefe(self, neueTiefe):
        """Veraendernde Methode fuer die Tiefe"""
        self.Loesche()
        self.__t = neueTiefe
        self.Zeichne()
    t = property(GibTiefe,AendereTiefe)


    ### w
    def GibWinkel(self):
        return self.__w
    def Drehe(self, drehWinkel):
        """Veraendernde Methode fuer den Winkel"""
        self.Loesche()
        self.__w += drehWinkel
        self.Zeichne()
    w = property(GibWinkel,Drehe)

    ### f
    def GibFarbe(self):
        return self.__f
    def AendereFarbe(self, neueFarbe):
        """Veraendernde Methode fuer die Farbe"""
        self.Loesche()
        self.__f = neueFarbe
        self.Zeichne()
    f = property(GibFarbe,AendereFarbe)

    ### s
    def IstSichtbar(self):
        return self.__s
    def AendereSichtbarkeit(self, sichtbar):
        """Veraendernde Methode fuer die Farbe"""
        if self.IstSichtbar and not(sichtbar):
            Zeichenflaeche.GibZeichenflaeche().Entferne(self)
        self.__s = sichtbar
        if sichtbar:
            Zeichenflaeche.GibZeichenflaeche().Zeichne(self)
    s = property(IstSichtbar,AendereSichtbarkeit)

        
    @abstractmethod
    def GibFigur(self):
        """definiert die zu zeichnende Figur (Pfad)"""
        return NotImplemented

    def Transformiere(self, path):
        '''transformiert den uebergebenen Pfad'''
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()

        x,y,b,t,w = self.__x, self.__y, self.__b, self.__t, self.__w
        gc.PushState()
        gc.Translate(x+b/2, y+t/2)
        gc.Rotate(radians(w))
        gc.Translate(-b/2, -t/2)
        transformation = gc.GetTransform()
        gc.PopState()
        path.Transform(transformation)
        return path

    def GibZeichenPfad(self):
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        return gc.CreatePath()

    ##### ----- erste beide Methoden besser nicht? -----
    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        self.AendereSichtbarkeit(False)

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        self.AendereSichtbarkeit(True)

    def Loesche(self):
        """Entfernt ein Objekt aus der Darstellung."""
        Zeichenflaeche.GibZeichenflaeche().Entferne(self)

    def Zeichne(self):
        """Fuegt das Objekt den darzustellenden hinzu."""
        Zeichenflaeche.GibZeichenflaeche().Zeichne(self)

### ---------------------------------------------------
class MoebelApp(wx.App):
    """Test-Anwendung speziell fuer Stuhl"""
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        self.TestAnwendung()
        return True

    def TestAnwendung(self):
        """Allein fuer die Testanwendung.
        Sollte nicht gehen! """
        try:
            moebel = Moebel(30, 30, 40, 40, 270, 'blue', True)
        except Exception as e:
            print( str(e) )

        
### ---------------------------------------------------
if __name__ == '__main__':
    app = MoebelApp(redirect=True)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
